@@ -27,36 +27,38 @@ class GroupCatchExceptions(click.Group):
             return self.main(*args, **kwargs)
 
         except apiclient.RequestMalformedException as e:
-            click.echo("ERROR: Malformed Request: %s" % error_text(e.text))
+            click.echo('ERROR: Malformed Request: %s' % error_text(e.text))
             sys.exit(1)
 
         except apiclient.UnauthorizedException:
-            click.echo("ERROR: Not authorized")
+            click.echo('ERROR: Not authorized')
             sys.exit(1)
 
         except apiclient.ResourceCannotBeDeletedException as e:
-            click.echo("ERROR: Cannot delete resource: %s" %
-                       error_text(e.text))
+            click.echo('ERROR: Cannot delete resource: %s' % error_text(e.text))
             sys.exit(1)
 
         except apiclient.ResourceNotFoundException as e:
-            click.echo("ERROR: Resource not found: %s" % error_text(e.text))
+            click.echo('ERROR: Resource not found: %s' % error_text(e.text))
             sys.exit(1)
 
         except apiclient.ResourceInUseException as e:
-            click.echo("ERROR: Resource in use:\n%s" % error_text(e.text))
+            click.echo('ERROR: Resource in use:\n%s' % error_text(e.text))
             sys.exit(1)
 
         except apiclient.InternalServerError as e:
             # Print full error since server should not fail
-            click.echo("ERROR: Internal Server Error:\n%s" % e.text)
+            click.echo('ERROR: Internal Server Error:\n%s' % e.text)
             sys.exit(1)
 
         except apiclient.InsufficientResourcesException as e:
-            click.echo("ERROR: Insufficient Resources:\n%s" %
+            click.echo('ERROR: Insufficient Resources:\n%s' %
                        error_text(e.text))
             sys.exit(1)
 
+        except apiclient.requests.exceptions.ConnectionError as e:
+            click.echo('ERROR: Unable to connect to server: %s' % e)
+
 
 def error_text(json_text):
     try:
@@ -74,7 +76,7 @@ def auto_complete(func):
 
     if not CLIENT:
         CLIENT = apiclient.Client(
-            namespace=os.getenv("SHAKENFIST_NAMESPACE"),
+            namespace=os.getenv('SHAKENFIST_NAMESPACE'),
             key=os.getenv('SHAKENFIST_KEY'),
             base_url=os.getenv('SHAKENFIST_API_URL', 'http://localhost:13000'),
         )
@@ -133,20 +135,22 @@ def node_list(ctx):
 
     if ctx.obj['OUTPUT'] == 'pretty':
         x = PrettyTable()
-        x.field_names = ['name', 'ip', 'lastseen']
+        x.field_names = ['name', 'ip', 'lastseen', 'version']
         for n in nodes:
-            x.add_row([n['name'], n['ip'], n['lastseen']])
+            x.add_row([n['name'], n['ip'], n['lastseen'], n['version']])
         print(x)
 
     elif ctx.obj['OUTPUT'] == 'simple':
-        print('name,ip,lastseen')
+        print('name,ip,lastseen,version')
         for n in nodes:
-            print('%s,%s,%s' % (n['name'], n['ip'], n['lastseen']))
+            print('%s,%s,%s,%s' % (
+                    n['name'], n['ip'], n['lastseen'], n['version']))
 
     elif ctx.obj['OUTPUT'] == 'json':
         filtered_nodes = []
         for n in nodes:
-            filtered_nodes.append(filter_dict(n, ['name', 'ip', 'lastseen']))
+            filtered_nodes.append(
+                filter_dict(n, ['name', 'ip', 'lastseen', 'version']))
         print(json.dumps({'nodes': filtered_nodes}, indent=4, sort_keys=True))
 
 
@@ -196,7 +200,7 @@ def namespace_create(ctx, namespace=None):
 
 @namespace.command(name='delete',
                    help=('delete a namespace.\n\n'
-                         'namespace: The name of the namespace'))
+                         'NAMESPACE: The name of the namespace'))
 @click.argument('namespace', type=click.STRING)
 @click.pass_context
 def namespace_delete(ctx, namespace=None):
@@ -249,6 +253,20 @@ def namespace_show(ctx, namespace=None):
     _show_namespace(ctx, namespace)
 
 
+@namespace.command(name='clean',
+                   help=('Clean (delete) namespace of all instances and networks'))
+@click.option('--confirm',  is_flag=True)
+@click.option('--namespace', type=click.STRING)
+@click.pass_context
+def namespace_clean(ctx, confirm=False, namespace=None):
+    if not confirm:
+        print('You must be sure. Use option --confirm.')
+        return
+
+    CLIENT.delete_all_instances(namespace)
+    CLIENT.delete_all_networks(namespace)
+
+
 @namespace.command(name='add-key',
                    help=('add a key to a namespace.\n\n'
                          'NAMESPACE: The name of the namespace\n'
@@ -421,6 +439,18 @@ def network_create(ctx, netblock=None, name=None, dhcp=None, nat=None, namespace
         netblock, dhcp, nat, name, namespace))
 
 
+@network.command(name='delete-all', help='Delete ALL networks')
+@click.option('--confirm',  is_flag=True)
+@click.option('--namespace', type=click.STRING)
+@click.pass_context
+def network_delete_all(ctx, confirm=False, namespace=None):
+    if not confirm:
+        print('You must be sure. Use option --confirm.')
+        return
+
+    CLIENT.delete_all_networks(namespace)
+
+
 @network.command(name='events', help='Display events for a network')
 @click.argument('network_uuid', type=click.STRING, autocompletion=_get_networks)
 @click.pass_context
@@ -543,9 +573,9 @@ def _show_instance(ctx, i, include_snapshots=False):
         snapshots = CLIENT.get_instance_snapshots(i['uuid'])
 
     if ctx.obj['OUTPUT'] == 'json':
-        out = filter_dict(i, ['uuid', 'name', 'namespace', 'cpus', 'memory', 'disk_spec',
-                              'node', 'console_port', 'vdi_port', 'ssh_key',
-                              'user_data'])
+        out = filter_dict(i, ['uuid', 'name', 'namespace', 'cpus', 'memory',
+                              'disk_spec', 'node', 'console_port', 'vdi_port',
+                              'ssh_key', 'user_data', 'power_state', 'state'])
         out['network_interfaces'] = []
         for interface in interfaces:
             _show_interface(ctx, interface, out)
@@ -572,6 +602,7 @@ def _show_instance(ctx, i, include_snapshots=False):
     print(format_string % ('memory', i['memory']))
     print(format_string % ('disk spec', i['disk_spec']))
     print(format_string % ('node', i['node']))
+    print(format_string % ('power state', i['power_state']))
     print(format_string % ('state', i['state']))
 
     # NOTE(mikal): I am not sure we should expose this, but it will do
@@ -713,8 +744,14 @@ def instance_create(ctx, name=None, cpus=None, memory=None, network=None, networ
     for d in disk:
         p = _parse_spec(d)
         size, base = p
+        try:
+            size_int = int(size)
+        except:
+            print('Disk size is not an integer')
+            return
+
         diskdefs.append({
-            'size': int(size),
+            'size': size_int,
             'base': base,
             'bus': None,
             'type': 'disk',
@@ -724,7 +761,8 @@ def instance_create(ctx, name=None, cpus=None, memory=None, network=None, networ
         for elem in d.split(','):
             s = elem.split('=')
             if len(s) != 2:
-                print('Error in disk specification - should be key=value: %s' % elem)
+                print('Error in disk specification -'
+                      ' should be key=value: %s' % elem)
                 return
             defn[s[0]] = s[1]
         diskdefs.append(defn)
@@ -743,7 +781,8 @@ def instance_create(ctx, name=None, cpus=None, memory=None, network=None, networ
         for elem in n.split(','):
             s = elem.split('=')
             if len(s) != 2:
-                print("Error in network specification - should be key=value: %s" % elem)
+                print('Error in network specification -'
+                      ' should be key=value: %s' % elem)
                 return
             defn[s[0]] = s[1]
         netdefs.append(defn)
@@ -764,6 +803,18 @@ def instance_delete(ctx, instance_uuid=None):
         print('{}')
 
 
+@instance.command(name='delete-all', help='Delete ALL instances')
+@click.option('--confirm',  is_flag=True)
+@click.option('--namespace', type=click.STRING)
+@click.pass_context
+def instance_delete_all(ctx, confirm=False, namespace=None):
+    if not confirm:
+        print('You must be sure. Use option --confirm.')
+        return
+
+    CLIENT.delete_all_instances(namespace)
+
+
 @instance.command(name='events', help='Display events for an instance')
 @click.argument('instance_uuid', type=click.STRING, autocompletion=_get_instances)
 @click.pass_context
