@@ -265,6 +265,7 @@ class Instance(object):
                     db.persist_ipmanager(ni['network_uuid'], ipm.save())
 
         db.update_instance_state(self.db_entry['uuid'], 'deleted')
+        db.update_instance_power_state(self.db_entry['uuid'], 'off')
         db.free_console_port(self.db_entry['console_port'])
         db.free_console_port(self.db_entry['vdi_port'])
 
@@ -459,6 +460,9 @@ class Instance(object):
             pass
 
         instance.setAutostart(1)
+        db.update_instance_power_state(
+            self.db_entry['uuid'],
+            util.extract_power_state(libvirt, instance))
 
     def power_off(self):
         libvirt = util.get_libvirt()
@@ -522,12 +526,20 @@ class Instance(object):
             instance.reset()
 
     def pause(self):
+        libvirt = util.get_libvirt()
         instance = self._get_domain()
         instance.suspend()
+        db.update_instance_power_state(
+            self.db_entry['uuid'],
+            util.extract_power_state(libvirt, instance))
 
     def unpause(self):
+        libvirt = util.get_libvirt()
         instance = self._get_domain()
         instance.resume()
+        db.update_instance_power_state(
+            self.db_entry['uuid'],
+            util.extract_power_state(libvirt, instance))
 
     def get_console_data(self, length):
         console_path = os.path.join(self.instance_path, 'console.log')
